using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.Primitives;

using Telerik.WinControls.UI;
using Telerik.WinControls.RadControlSpy;

namespace CarouselImageGallery
{
    public partial class GalleryView : RadForm
    {
        private bool shouldZoomSelectedItem = false;
        private CarouselContentItem zoomedItem;
        private float currScaleFactor = 1f;
        private int animationRunning;

        public GalleryView()
        {
            InitializeComponent();
            new RadControlSpyForm().Show();
            this.radCarousel1.ZoomGesture += new ZoomGestureEventHandler(radCarousel1_ZoomGesture);
            this.radCarousel1.RotateGesture += new RotateGestureEventHandler(radCarousel1_RotateGesture);
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            this.radCarousel1.EnableGesture(GestureType.Zoom);
            this.radCarousel1.EnableGesture(GestureType.Rotate);
        }

        private void radCarousel1_ZoomGesture(object sender, ZoomGestureEventArgs e)
        {
            if (this.zoomedItem == null) return;

            //reset the animation setting and set is as local value:
            this.zoomedItem.SetValue(RadElement.ScaleTransformProperty, this.zoomedItem.ScaleTransform);
            this.zoomedItem.ResetValue(RadElement.ScaleTransformProperty, ValueResetFlags.Animation);

            this.zoomedItem.ScaleTransform = new SizeF(
                (float)(zoomedItem.ScaleTransform.Width * e.ZoomFactor),
                (float)(zoomedItem.ScaleTransform.Height * e.ZoomFactor));
        }

        private void radCarousel1_RotateGesture(object sender, RotateGestureEventArgs e)
        {
            if (this.zoomedItem == null) return;
            zoomedItem.AngleTransform -= (float)(e.Angle * 180D / Math.PI); 

        }

        public void ShowGallery(string path)
        {
            this.Text = path;

            string[] jpgFiles = Directory.GetFiles(path, "*.jpg", SearchOption.AllDirectories);
            string[] jpegFiles = Directory.GetFiles(path, "*.jpeg", SearchOption.AllDirectories);
            string[] bmpFiles = Directory.GetFiles(path, "*.bmp", SearchOption.AllDirectories);
            string[] gifFiles = Directory.GetFiles(path, "*.gif", SearchOption.AllDirectories);
            string[] pngFiles = Directory.GetFiles(path, "*.png", SearchOption.AllDirectories);

            this.radCarousel1.Items.Clear();

            List<string> imageFiles = new List<string>(jpgFiles.Length + jpegFiles.Length + bmpFiles.Length + gifFiles.Length + pngFiles.Length);
            imageFiles.AddRange(jpgFiles);
            imageFiles.AddRange(jpegFiles);
            imageFiles.AddRange(bmpFiles);
            imageFiles.AddRange(gifFiles);
            imageFiles.AddRange(pngFiles);

            this.radCarousel1.DataSource = imageFiles;
            this.Show();
        }

        private void radCarousel1_ItemDataBound(object sender, Telerik.WinControls.UI.ItemDataBoundEventArgs e)
        {
            RadButtonElement button = (RadButtonElement)e.DataBoundItem;
            button.Text = (string)e.DataItem;
            button.MinSize = new Size(20, 20);
            //button.AutoSize = false;
            button.DisplayStyle = DisplayStyle.Image;
            button.ImagePrimitive.ImageLayout = ImageLayout.Zoom;

            button.MouseDown += new MouseEventHandler(button_MouseDown);

            if (this.radCarousel1.Items.Count < this.radCarousel1.VisibleItemCount)
                LoadItemImage(button, true);
        }

        private void button_MouseDown(object sender, MouseEventArgs e)
        {
            CarouselContentItem currentZoomed = zoomedItem;
            ResetZoomedItem();

            RadButtonElement button = (RadButtonElement)sender;

            CarouselContentItem contentItem = (CarouselContentItem)button.Parent;
            if (currentZoomed == contentItem)
                return;

            if (this.radCarousel1.SelectedItem != button)
                return;

            ZoomSelectedItemOnAnimationFinished();
        }

        private void ZoomItem(RadButtonElement button)
        {
            shouldZoomSelectedItem = false;
            Size size = this.radCarousel1.CarouselElement.ItemsContainer.Size;

            CarouselContentItem contentItem = (CarouselContentItem)button.Parent;

            zoomedItem = contentItem;

            button.ButtonFillElement.Visibility = ElementVisibility.Collapsed;

            currScaleFactor = Math.Min(
                ((float)size.Width) / button.Size.Width / 2f,
                ((float)size.Height) / button.Size.Height / 2f);

            AnimatedPropertySetting scaleChange = new AnimatedPropertySetting(
                RadElement.ScaleTransformProperty,
                contentItem.ScaleTransform,
                new SizeF(currScaleFactor, currScaleFactor),
                5,
                20
                ); 

            scaleChange.AnimationFinished += new AnimationFinishedEventHandler(scaleChange_AnimationFinished);

            SizeF offset = new SizeF(
                (size.Width - contentItem.Size.Width * currScaleFactor) / 2f - contentItem.Location.X,
                (size.Height - contentItem.Size.Height * 0.50f * currScaleFactor) / 2f - contentItem.Location.Y);

            AnimatedPropertySetting positionChange = new AnimatedPropertySetting(
                RadElement.PositionOffsetProperty,
                contentItem.PositionOffset,
                offset,
                5,
                20
                );

            scaleChange.ApplyValue(contentItem);
            positionChange.ApplyValue(contentItem);
        }

        private void scaleChange_AnimationFinished(object sender, AnimationStatusEventArgs e)
        {
            RadButtonElement button = (RadButtonElement)this.zoomedItem.HostedItem;
            LoadItemImage(button, false);
            //SizeToFit is needed in order to enable ScaleSize
            button.ImagePrimitive.ImageScaling = Telerik.WinControls.Enumerations.ImageScaling.SizeToFit;
            button.ImagePrimitive.ScaleSize = new Size(
            (int)Math.Round(button.Size.Width - button.BorderThickness.All * currScaleFactor / 2f),
            (int)Math.Round(button.Size.Height - button.BorderThickness.All * currScaleFactor / 2f));
        }

        private bool GetThumbnailImageAbort()
        {
            return false;
        }

        private void GalleryView_Shown(object sender, EventArgs e)
        {
            if (this.radCarousel1.Items.Count > 0)
            {
                this.radCarousel1.SelectedIndex = 0;
            }

            this.Text = string.Format("Showing gallery of {0} images. Location: ", this.radCarousel1.Items.Count) + this.Text;

            this.radCarousel1.CarouselElement.ItemsContainer.ForceUpdate();
        }

        private void radCarousel1_NewCarouselItemCreating(object sender, NewCarouselItemCreatingEventArgs e)
        {
            e.NewCarouselItem = new RadButtonElement();
        }

        private void GalleryView_Load(object sender, EventArgs e)
        {
            //this.radCarousel1.EnableAutoLoop = true;
        }

        private void radCarousel1_CarouselElement_ItemEntering(object sender, ItemEnteringEventArgs e)
        {
            RadButtonElement button = (RadButtonElement)this.radCarousel1.Items[e.ItemIndex];
            LoadItemImage(button, true);
        }

        private void LoadItemImage(RadButtonElement button, bool thumbnail)
        {
            if (button != null && !string.IsNullOrEmpty(button.Text) &&
                (button.Image == null || !thumbnail))
            {
                Image res = Image.FromFile(button.Text);

                button.Image = thumbnail
                                   ?
                                       res.GetThumbnailImage((int)(70d * ((double)res.Width / res.Height)),
                                                             70, this.GetThumbnailImageAbort,
                                                             IntPtr.Zero)
                                   :
                                       res;
            }
        }

        private void radCarousel1_CarouselElement_ItemLeaving(object sender, ItemLeavingEventArgs e)
        {
            RadButtonElement button = (RadButtonElement)this.radCarousel1.Items[e.ItemIndex];
            UnloadItemImage(button);
        }

        private void UnloadItemImage(RadButtonElement button)
        {
            if (button != null && button.Image != null)
            {
                button.Image = null;
            }
        }

        private void ResetZoomedItem()
        {
            if (this.zoomedItem == null)
                return;

            ((RadButtonElement)zoomedItem.HostedItem).ButtonFillElement.Visibility = ElementVisibility.Visible;
            //reset the scale size
            ((RadButtonElement)zoomedItem.HostedItem).ImagePrimitive.ImageScaling = Telerik.WinControls.Enumerations.ImageScaling.None;

            AnimatedPropertySetting scaleChange = new AnimatedPropertySetting(
              RadElement.ScaleTransformProperty,
              this.zoomedItem.ScaleTransform,
              new SizeF(1f, 1f),
              5,
              20
              );

            AnimatedPropertySetting positionChange = new AnimatedPropertySetting(
                RadElement.PositionOffsetProperty,
                this.zoomedItem.PositionOffset,
                new SizeF(0, 0),
                5,
                20
                );

            AnimatedPropertySetting rotationChange = new AnimatedPropertySetting(
                RadElement.AngleTransformProperty,
                this.zoomedItem.AngleTransform,
                0.0f,
                5,
                20
                );

            scaleChange.ApplyValue(zoomedItem);
            positionChange.ApplyValue(zoomedItem);
            rotationChange.ApplyValue(zoomedItem);

            ((RadButtonElement)zoomedItem.HostedItem).ImagePrimitive.ScaleSize = Size.Empty;

            UnloadItemImage((RadButtonElement)zoomedItem.HostedItem);
            LoadItemImage((RadButtonElement)zoomedItem.HostedItem, true);

            this.zoomedItem = null;
        }

        private void radCarousel1_SelectedIndexChanged(object sender, EventArgs e)
        {
            ResetZoomedItem();
        }

        private void radCarousel1_CarouselElement_AnimationFinished(object sender, EventArgs e)
        {
            this.animationRunning--;

            if (animationRunning < 0)
                animationRunning = 0;

            if (this.shouldZoomSelectedItem)
                ZoomSelectedItemOnAnimationFinished();
        }

        private void ZoomSelectedItemOnAnimationFinished()
        {
            shouldZoomSelectedItem = true;

            if (this.radCarousel1.SelectedItem != null && animationRunning == 0)
            {
                this.ZoomItem((RadButtonElement)this.radCarousel1.SelectedItem);
            }
        }

        private void radCarousel1_CarouselElement_AnimationStarted(object sender, EventArgs e)
        {
            this.animationRunning++;
            this.ResetZoomedItem();
        }
    }
}